#!/bin/sh
exec ${GUILE-guile} -e main -s $0 "$@"
!#
;; A <section_name>...</section_name> pair that will appear in the title and
;; the h1.
;;
;; A single <local_nav> </local_nav> pair that gets substitued into the page,
;; and a single <main_content> </main_content> pair.
;;
;; The tempate file has to have local_nav appear before main_content.  Both
;; must be present.  On the template side, there must be no content between the
;; tags that the victim will populate.
;;
;; The first argument is the template file, the second is the file it's to be
;; applied to.  Output is written to stdout.

;; based on template.pl by Peter C Norton
;; reworked by Jorgen Schaefer and Thien-Thi Nguyen

(use-modules (ice-9 regex)
             (ice-9 rdelim))

(define (main args)
  (if (not (= 3 (length args)))
      (error "usage: mkpage <template> <datafile>"))
  (let ((template (open-input-file (list-ref args 1)))
        (datafile (open-input-file (list-ref args 2))))
    (display (apply apply-data
                    (read-complete-file template)
                    (read-datafile datafile)))))

(define (read-complete-file port)
  (read-delimited "" port))

;; Returns a list (section-name local-nav main-content)
;; Could use multiple return values instead...
(define (read-datafile port)
  (let* ((data (read-complete-file port))
         (extract (lambda (tag)
                    (let ((full (format #f "<~A>(.*)</~A>" tag tag)))
                      (cond ((string-match full data)
                             => (lambda (match)
                                  (match:substring match 1)))
                            (else ""))))))
    (map extract '(section_name local_nav main_content))))

(define (apply-data template section-name local-nav main-content)
  (substitute-match
   (make-regexp "<section_name>[^<]*</section_name>")
   section-name
   (substitute-match
    (make-regexp "<local_nav>[^<]*</local_nav>")
    local-nav
    (substitute-match
     (make-regexp "<main_content>[^<]*</main_content>")
     main-content
     template))))

(define (substitute-match rx replacement data)
  (let ((match (regexp-exec rx data)))
    (if (not match)
        data
        (string-append ;;; inefficient, but we can live with that
         (match:prefix match)
         replacement
         (substitute-match rx replacement (match:suffix match))))))

;;; template.scm ends here
